package code;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

import db.DBC;
import db.Dominio;

public class InstallHandler implements HttpHandler {

	@Override
	public void handle(HttpExchange he) throws IOException {

		String requestMethod = he.getRequestMethod();

		if (requestMethod.compareToIgnoreCase("options") == 0) {
			Helper.sendCors(he, 200);
			return;
		}

		if (requestMethod.compareToIgnoreCase("POST") != 0) {
			Helper.sendCors(he, 405);
			return;
		}
		String body = Helper.readBody(he.getRequestBody());

		String user;

		if ((user = Helper.checkTokenGetUser(he)) == null) {
			Helper.sendCors(he, 401);
			return;
		}

		// verifica user


		JSONObject j = null;
		try {
			j = new JSONObject(body);
			String dm = j.getString("domain");

			// try {

			Dominio d = DBC.getDom(dm);
			if (d != null) {
				Helper.sendCors(he, 403);
				return;
			}

		} catch (JSONException | SQLException e) {
			e.printStackTrace();
			return;
		}

		// inserisco i dati nel DB

		// String s = user + "-A";
		try {

			// qui leggo e parsifico i json nel body, inserisco tutti i campi nel db

			String domain = j.getString("domain");
			DBC.insertDom(domain);
			// questa chiamata serve ad assicurarsi che l'utente che ha effettuato la
			// chiamata sia anche amministratore del dominio
			DBC.insertAmministra(user, domain);

			JSONArray arrUsers = j.getJSONArray("users");
			for (int i = 0; i < arrUsers.length(); i++) {
				JSONObject userObj = arrUsers.getJSONObject(i);
				String usr = userObj.getString("user");
				if (user.equals(usr))
					continue;
				if (userObj.getString("role").equals("A")) {
					DBC.insertAmministra(usr, domain);
				} else if (userObj.getString("role").equals("U")) {
					DBC.insertUsa(usr, domain);
				} else
					System.err.println(userObj.toString());
			}

			JSONArray arrServ = j.getJSONArray("services");// array con solo il nome dei servizi da installare
			ArrayList<String> moduleHosts = new ArrayList<String>();
			for (int i = 0; i < arrServ.length(); i++) {

				// String modul=((JSONObject) arrUsers.get(i)).getString("service");
				// String host=((JSONObject) arrUsers.get(i)).getString("host");
				// DBC.insertService(domain,host,modul);
				String modul = arrServ.getString(i);
				moduleHosts.addAll(DBC.insertService(domain, modul));
			}
			// divido gli host in base al loro module
			HashMap<String, ArrayList<String>> hostsMap = new HashMap<String, ArrayList<String>>();
			for (String mh : moduleHosts) {
				String[] mhSplit = mh.split("---");
				String module = mhSplit[0];
				String host = mhSplit[1];
				ArrayList<String> hostsList;
				if (hostsMap.containsKey(module)) {
					hostsList = hostsMap.get(module);
				} else {
					hostsList = new ArrayList<String>();
					hostsMap.put(module, hostsList);
				}
				hostsList.add(host);
			}
			// imposta correttamente l'array dei servizi
			JSONArray arrServiziFinale = new JSONArray();
			for (int i = 0; i < arrServ.length(); i++) {
				JSONObject row = new JSONObject();
				String mod = arrServ.getString(i);
				ArrayList<String> ho = hostsMap.get(mod);
				for (String h : ho) {
					row.put("host", h);
					row.put("service", mod);
					row.put("uri", DBC.getURI(mod));
					arrServiziFinale.put(row);
				}
			}
			j.put("services", arrServiziFinale);
			// non ci sono controlli!!!!

		} catch (SQLException | JSONException e) {
			e.printStackTrace();
		}

		// ora bisogna fare la chiamata al CloudApp, non prima!
		// effettuo chiamata a CloudAppManager
		// preso da https://www.baeldung.com/java-http-request

		// è una chiamata annidata nella risposta alla webapp
		// -richiesta REST da webApp a /install
		// -prendo da DB e poi chiamo CloudAppMng su /install
		// -attendo risposta da CloudAppMng e chiudo
		// -rispondo a webApp e chiudo
		// EZ

		//
		// standard per chiamata in slide
		// https://www.dir.uniupo.it/pluginfile.php/948883/mod_resource/content/1/FrameworkProgetto5.pdf
		//
		// http://127.0.0.1:8080/install

		// 3002/install da inserire
		/*
		 * URL url = new URL(Helper.getCloudappURL()+"install");// maybe, se CloudAppe è
		 * in localhost porta 8080
		 * HttpURLConnection con = (HttpURLConnection) url.openConnection();
		 *
		 * con.setRequestMethod("POST");
		 * con.setRequestProperty("Content-Type", "application/json");
		 * con.setRequestProperty("Accept", "application/json");
		 * con.setDoOutput(true);
		 * con.setConnectTimeout(5000);
		 * con.setReadTimeout(5000);
		 * DataOutputStream out = new DataOutputStream(con.getOutputStream());//
		 * inserimento param
		 * // in call
		 * out.writeBytes(j.toString());
		 * out.flush();
		 * out.close();
		 *
		 * // con.setRequestProperty("Content-Type", "application/json");
		 * // String contentType = con.getHeaderField("Content-Type");
		 * // leggo risposta
		 * int status = con.getResponseCode();
		 * BufferedReader in = new BufferedReader(new
		 * InputStreamReader(con.getInputStream()));
		 * String inputLine;
		 * StringBuffer content = new StringBuffer();
		 * while ((inputLine = in.readLine()) != null) {
		 * content.append(inputLine);
		 * }
		 * in.close();
		 */
		HttpURLConnection con = Helper.sendMessageToCloudapp("install", j.toString());
		int status = con.getResponseCode();
		// FIXME serve avere anche il content?
		String cloudappResponse = Helper.getResponseFromConnection(con);
		con.disconnect();

		// finita chiamata a CloudApp

		// f.close();
		// risposta alla webapp
		Helper.sendCors(he, status);

	}

}
